import _ from 'lodash'

import { GitlabApi } from './gitlab.api'
import { GitEnricherInputs } from '../../../image-enricher-git-info/types'

export class GitlabService {
    private gitlabApi

    constructor(inputs: GitEnricherInputs) {
        this.gitlabApi = new GitlabApi(inputs)
    }

    async getPullRequestsWithCommits(repo, branch) {
        try {
            const prs = await this.gitlabApi.getPullRequests(repo, branch)
            return Promise.all(prs.map(async (pr) => {
                const info = await this.gitlabApi.getCommitsInfo(repo, pr)
                const result = {
                    ...info,
                    prDate: pr.closed_at || pr.updated_at,
                    number: pr.iid,
                    url: `${pr.web_url}`,
                    title: pr.title,
                }

                return result
            }))
        } catch (error) {
            throw new Error(`failed to get pull requests: ${error.message}`)
        }
    }

    async getBranch(repo, branch) {
        try {
            const gitlabBranch = await this.gitlabApi.getBranch(repo, branch)
            return {
                name: gitlabBranch.name,
                commit: gitlabBranch.commit.id,
                commitMsg: gitlabBranch.commit.message,
                commitURL: gitlabBranch.commit.web_url || gitlabBranch.web_url,
                committerUsername: gitlabBranch.commit.committer_name
            }
        } catch (error) {
            if (_.get(error, 'response.statusCode') === 404) {
                // branch could be delete, but PR still can be found
                return
            }
            throw new Error(`failed to get branch: ${error.message}`)
        }
    }
}
